#include "StdAfx.h"
#include "WASListTenantDomain.h"
#include "ConvertUTF.h"
#include "MessengerActor.h"
#include "UtilFunction.h"
#include "GameTalkAPIGlobal.h"


CWASListTenantDomain::CWASListTenantDomain(CWASActor *pActor) : 
	CWASCommand(pActor)
{
}


CWASListTenantDomain::~CWASListTenantDomain(void)
{
}


void CWASListTenantDomain::ExecuteAsync()
{
	m_listDomain.clear();

	CMessengerActor *pMessengerActor = static_cast<CMessengerActor *>(m_pActor->GetMessengerActor());
	if(pMessengerActor == NULL)
	{
		return;
	}	

	USES_CONVERSION;
	USES_UTF8_CONVERSION;

	WSTRING strURL = m_pActor->GetWASServerAddress();
	strURL += _T("/tenant/ListDomain.php");

	m_httpFacade.session().SetURL( T2CA(strURL.c_str()), pMessengerActor->GetMessengerTicket());
	m_httpFacade.session().SetMethod(HTTP_METHOD_POST);

	m_httpFacade.session().AddBodyVariable("tenant", W2U8(m_strTenant.c_str()));

	m_httpFacade.session().SetKeepRawHeader(TRUE);
	m_httpFacade.SendRequest(this);
}


bool CWASListTenantDomain::ParseAndEventFire(nfc::CIkXMLDom * pDom)
{
	if(pDom == NULL)
		return false;

	nfc::IIkXMLNode* pNode = pDom->GetdocumentElement()->selectSingleNode("nodes");
	if (pNode == NULL)
	{
		m_strWasErrMsg += _T("domain list  \"<nodes>\" ã  ϴ.");
		return false;
	}

	nfc::IIkXMLNodeList *pList = pNode->selectNodes("node");	
	nfc::IIkXMLAttribute* pAttr;
	USES_CONVERSION;
	USES_UTF8_CONVERSION;

	for (int i = 0; i < pList->Getlength(); i++)
	{	
		pNode = pList->Getitem(i);
		GT_ASSERT(pNode);

		if (NULL == (pAttr = pNode->getAttribute("domain_name")))// ʼ
			continue;	// ignore this item

		//sAddrPrivate.nID = atoi(pAttr->Gettext().c_str());
		GT_TENANT_DOMAIN_DATA sDomainData;
		sDomainData.strDomain = U82W(pAttr->Gettext().c_str());
		m_listDomain.push_back(sDomainData);
	}

	return true;
}


// override in drived class
void CWASListTenantDomain::OnWasResponse(trid_t trid,
								  bool bSuccess,
								  int /*nWinINetErrorCode*/, 
								  int /*nHttpResponseCode*/, 
                                  LPCSTR /*szWasResCodeInHeader*/, 
                                  LPCSTR szWasResCodeInXml,
                                  nfc::CIkXMLDom * pDom)
{
	if(bSuccess)
	{
		if(ParseAndEventFire(pDom) == false)
			m_eError = wasError_Protocol;
	}

	GT_ASSERT(m_pActor);

	//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
	//EnterCriticalSection(pCS);

	CGTObserved<IGTWASActorEvent>::listobserver_t observers;
	m_pActor->GetObservers(&observers);
	CGTObserved<IGTWASActorEvent>::listobserver_t::iterator it = observers.begin();

    GT_TENANT_DOMAIN_DATA_LIST sList;
	sList.nSize = m_listDomain.size();
	sList.pList = sList.nSize > 0 ? &m_listDomain[0] : NULL;

	for (; it != observers.end(); ++it)
	{
		IGTWASActorEvent *pObserver = static_cast<IGTWASActorEvent *>(*it);
		if(pObserver->OnGTWASActor_ResponseTenantDomainList(m_pActor, GetID(), m_eError, sList) == false)
			break;
	}

	//LeaveCriticalSection(pCS);

	if(m_pActor)
		m_pActor->ReleaseCommand(GetID());
}


void CWASListTenantDomain::OnRequestTimeout(CHttpFacade * /*pHttpF*/, DWORD /*dwSeconds*/)
{
	m_eError = wasError_Timeout;

	GT_ASSERT(m_pActor);

	//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
	//EnterCriticalSection(pCS);

	CGTObserved<IGTWASActorEvent>::listobserver_t observers;
	m_pActor->GetObservers(&observers);
	CGTObserved<IGTWASActorEvent>::listobserver_t::iterator it = observers.begin();

	GT_TENANT_DOMAIN_DATA_LIST sList;
	sList.nSize = m_listDomain.size();
	sList.pList = sList.nSize > 0 ? &m_listDomain[0] : NULL;

	for (; it != observers.end(); ++it)
	{
		IGTWASActorEvent *pObserver = static_cast<IGTWASActorEvent *>(*it);
		if(pObserver->OnGTWASActor_ResponseTenantDomainList(m_pActor, GetID(), m_eError, sList) == false)
			break;
	}

	//LeaveCriticalSection(pCS);

	if(m_pActor)
		m_pActor->ReleaseCommand(GetID());
}
